/*
    DSCompress v1.0a
    Author: carpman <carpman@voidptr.org>
    http://voidptr.org

    Copyright 2007 Daniel Jackson

    This file is part of DSCompress.

    DSCompress is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    DSCOmpress is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "linkedlist.h"

ctLList *newLList(void){
  ctLList *newList = (ctLList *)malloc(sizeof(ctLList));

  newList->headElement = NULL;
  newList->endJump = NULL;
  newList->elementCount = 0;

  return newList;
}

void freeLList(ctLList *list){
  ctLListElement *currentElement = list->headElement;

  while(currentElement != NULL){
    ctLListElement *oldElement = currentElement;
    free(currentElement->data);
    currentElement = currentElement->next;
    free(oldElement);
  }

  free(list);
}

void appendLListElement(ctLList *list, void *data, long dataSize){
  ctLListElement *newElement = malloc(sizeof(ctLListElement));

  newElement->data = malloc(dataSize);
  memcpy(newElement->data, data, dataSize);
  newElement->dataSize = dataSize;

  if(list->headElement == NULL){
    list->headElement = newElement;
    list->endJump = newElement;
    newElement->next = NULL;
    newElement->previous = NULL;
  }else{
    list->endJump->next = newElement;
    newElement->previous = list->endJump;
    list->endJump = newElement;
    newElement->next = NULL;
  }

  list->elementCount++;
}

void insertLListElement(ctLList *list, long index, void *data, long dataSize){
  ctLListElement *newElement = malloc(sizeof(ctLListElement));
  ctLListElement *currentElement = list->headElement;
  int loopCount = 0;

  newElement->data = malloc(dataSize);
  memcpy(newElement->data, data, dataSize);
  newElement->dataSize = dataSize;

  if(index == 0){
    ctLListElement *oldHead = list->headElement;
    list->headElement = newElement;
    newElement->next = oldHead;
    oldHead->previous = newElement;
  }else{
    ctLListElement *oldElement;

    for(loopCount = 0; loopCount < index; loopCount++){
      currentElement = currentElement->next;
    }

    oldElement = currentElement;

    if(currentElement->next == NULL){
      currentElement->next = newElement;
      newElement->previous = currentElement;
      list->endJump = newElement;
    }else{
      newElement->next = currentElement->next;
      currentElement->next = newElement;
      ((ctLListElement *)newElement->next)->previous = newElement;
      newElement->previous = currentElement;
    }
  }
    list->elementCount++;
}

void removeLListElement(ctLList *list, long index){
  ctLListElement *currentElement = list->headElement;
  int loopCount = 0;

  if(index >= list->elementCount)
    return;

  if(index == 0){
    ctLListElement *oldElement = currentElement;
    list->headElement = currentElement->next;
    if(list->headElement != NULL)
      list->headElement->previous = NULL;
    free(oldElement->data);
    free(oldElement);
  }else{
    ctLListElement *oldElement;

    for(loopCount = 0; loopCount < index; loopCount++){
      currentElement = currentElement->next;
    }
    
    oldElement = currentElement;

    if(currentElement->next == NULL){
      ((ctLListElement *)currentElement->previous)->next = NULL;
    }else{
      ((ctLListElement *)currentElement->previous)->next = (ctLListElement *)currentElement->next;
      ((ctLListElement *)currentElement->next)->previous = (ctLListElement *)currentElement->previous;
    }
    free(oldElement->data);
    free(oldElement);
  }

  list->elementCount--;
} 

long getLListSize(ctLList *list){
  return list->elementCount;
}

void *getLListElement(ctLList *list, long index, long *dataSize){
  ctLListElement *currentElement = list->headElement;
  int loopCount = 0;

  if(index >= list->elementCount)
    return NULL;

  for(loopCount = 0; loopCount < index; loopCount++){
    currentElement = currentElement->next;
  }

  *dataSize = currentElement->dataSize;
  return currentElement->data;
}

ctLListIterator *getLListIterator(ctLList *list){
  ctLListIterator *newIterator = malloc(sizeof(ctLListIterator));

  newIterator->list = list;
  newIterator->currentElement = list->headElement;

  return newIterator;
}

void freeLListIterator(ctLListIterator *iterator){
  free(iterator);
}

void *nextLListIterator(ctLListIterator *iterator, long *dataSize){
  void *data = NULL;

  if(iterator->currentElement == NULL)
    return NULL;

  data = iterator->currentElement->data;
  
  *dataSize = iterator->currentElement->dataSize;
  iterator->currentElement = iterator->currentElement->next;

  return data;
}

int haveNextLListIterator(ctLListIterator *iterator){
  return iterator->currentElement->next == NULL;
}

void resetLListIterator(ctLListIterator *iterator){
  iterator->currentElement = iterator->list->headElement;
}
